from math import exp
from pathlib import Path
from argparse import ArgumentParser
from msdsl import MixedSignalModel, VerilogGenerator

def main():
    print('Running model generator...')

    # parse command line arguments
    parser = ArgumentParser()
    parser.add_argument('-o', '--output', type=str, default='build')
    parser.add_argument('--dt', type=float, default=0.1e-6)
    parser.add_argument('--tau', type=float, default=1.0e-6)
    a = parser.parse_args()

    # create the model
    m = MixedSignalModel('model', dt=a.dt)
    m.add_analog_input('v_in')
    m.add_analog_output('v_out')

    # apply dynamics
    # TODO: clean up with update to MSDSL
    class ce:
        name = '`CKE_MSDSL'
    m.set_next_cycle(m.v_out, m.v_out*exp(-a.dt/a.tau) + m.v_in*(1-exp(-a.dt/a.tau)), ce=ce)

    # determine the output filename
    filename = Path(a.output).resolve() / f'{m.module_name}.sv'
    print(f'Model will be written to: {filename}')

    # generate the model
    m.compile_to_file(VerilogGenerator(), filename)

if __name__ == '__main__':
    main()



import pytest
import importlib
import numpy as np
from pathlib import Path

root = Path(__file__).resolve().parent

from ..common import run_simulation, run_emulation, CommonArgParser, DEFAULT_SIMULATOR

def test_func_sim(simulator_name=DEFAULT_SIMULATOR):
    run_simulation(root=root, simulator_name=simulator_name)

@pytest.mark.skip(
   reason="This test takes a long time to run and largely covers the same features as test_rc."
)
@pytest.mark.skipif(not importlib.util.find_spec("cvxpy"),
                    reason="cvxpy is not available in python distribution")
def test_func_emu(gen_bitstream=True, emulate=True):
    run_emulation(root=root, gen_bitstream=gen_bitstream,
                  emu_ctrl_fun=emu_ctrl_fun, emulate=emulate)

def emu_ctrl_fun(ctrl):
    # reset emulator
    ctrl.set_reset(1)
    ctrl.set_reset(0)

    # reset everything else
    ctrl.set_param(name='in_', value=0.0)

    # save the outputs
    inpts = np.random.uniform(-1.2 * np.pi, +1.2 * np.pi, 100)
    apprx = np.zeros(shape=inpts.shape, dtype=float)
    for k, in_ in enumerate(inpts):
        ctrl.set_param(name='in_', value=in_)
        ctrl.refresh_param('vio_0_i')
        apprx[k] = ctrl.get_param('out')

    # compute the exact response to inputs
    def myfunc(x):
        # clip input
        x = np.clip(x, -np.pi, +np.pi)
        # apply function
        return np.sin(x)
    exact = myfunc(inpts)

    # check the result
    err = np.sqrt(np.mean((exact - apprx)**2))
    assert err <= 0.001

if __name__ == '__main__':
    CommonArgParser(sim_fun=test_func_sim, emu_fun=test_func_emu)



import numpy as np
from pathlib import Path
from argparse import ArgumentParser
from msdsl import MixedSignalModel, VerilogGenerator

def myfunc(x):
    # clip input
    x = np.clip(x, -np.pi, +np.pi)
    # apply function
    return np.sin(x)

def gen_model(order, numel, build_dir):
    # settings:
    # order=0, numel=512 => rms_error <= 0.0600
    # order=1, numel=128 => rms_error <= 0.0012
    # order=2, numel= 32 => rms_error <= 0.0010

    # create mixed-signal model
    m = MixedSignalModel('model', build_dir=build_dir)
    m.add_analog_input('in_')
    m.add_analog_output('out')

    # create function
    real_func = m.make_function(myfunc, domain=[-np.pi, +np.pi],
                                order=order, numel=numel)

    # apply function
    # TODO: clean up with update to MSDSL
    class ce:
        name = '`CKE_MSDSL'
    m.set_from_sync_func(m.out, real_func, m.in_, ce=ce)

    # write the model
    return m.compile_to_file(VerilogGenerator())

def main():
    print('Running model generator...')

    # parse command line arguments
    parser = ArgumentParser()
    parser.add_argument('-o', '--output', type=str, default='build')
    parser.add_argument('--dt', type=float, default=0.1e-6)
    parser.add_argument('--order', type=int, default=2)
    parser.add_argument('--numel', type=int, default=32)

    # parse arguments
    args = parser.parse_args()

    # call the model generator
    build_dir = Path(args.output).resolve()
    gen_model(order=args.order, numel=args.numel, build_dir=build_dir)

if __name__ == '__main__':
    main()



import os
import numpy as np
import scipy.interpolate
from argparse import ArgumentParser
from anasymod.analysis import Analysis


DEFAULT_SIMULATOR = 'icarus' if 'FPGA_SERVER' not in os.environ else 'vivado'


def run_simulation(root, simulator_name):
    # create analysis object
    ana = Analysis(input=root,
                   simulator_name=simulator_name)

    # generate functional models
    ana.gen_sources()

    # run the simulation
    ana.simulate()

    # print string to indicate test is done
    print('Done.')


def run_emulation(root, gen_bitstream=False, emu_ctrl_fun=None, emulate=False):
    # create analysis object
    ana = Analysis(input=root)

    # generate functional models
    ana.gen_sources()
    ana.set_target(target_name='fpga')  # set the active target to 'fpga'

    # generate the bitstream if desired
    if gen_bitstream:
        ana.build()  # generate bitstream for project

    # run the specific emulation sequence if desired
    if emulate and (emu_ctrl_fun is not None):
        ctrl = ana.launch(debug=True)  # start interactive control
        emu_ctrl_fun(ctrl)

    # print string to indicate test is done
    print('Done.')


class CommonArgParser(ArgumentParser):
    def __init__(self, sim_fun, emu_fun, *args, **kwargs):
        # call the super constructor
        super().__init__(*args, **kwargs)

        # parse command-line arguments
        self.add_argument('--sim', action='store_true')
        self.add_argument('--emulate', action='store_true')
        self.add_argument('--gen_bitstream', action='store_true')
        self.add_argument('--simulator_name', type=str, default=DEFAULT_SIMULATOR)
        args = self.parse_args()

        # run actions as requested
        if args.sim:
            print('Running simulation...')
            sim_fun(simulator_name=args.simulator_name)
        if args.gen_bitstream or args.emulate:
            print('Running emulation...')
            emu_fun(gen_bitstream=args.gen_bitstream, emulate=args.emulate)


class Waveform:
    def __init__(self, data, time):
        self.data = np.array(data, dtype=float)
        self.time = np.array(time, dtype=float)
        self.interp = scipy.interpolate.interp1d(
            time, data,
            bounds_error=False, fill_value=(data[0], data[-1])
        )

    def check_in_limits(self, wave_lo, wave_hi):
        # compute bounds
        lo_bnds = wave_lo.interp(self.time)
        hi_bnds = wave_hi.interp(self.time)

        # import matplotlib.pyplot as plt
        # plt.plot(self.time, lo_bnds)
        # plt.plot(self.time, hi_bnds)
        # plt.plot(self.time, self.data)
        # plt.legend(['lo_bnds', 'hi_bnds', 'data'])
        # plt.show()

        # find any point where the waveform is out of spec
        in_spec = np.logical_and(lo_bnds <= self.data, self.data <= hi_bnds)
        if np.all(in_spec):
            pass
        else:
            indices = np.where(np.logical_not(in_spec))
            times = self.time[indices]
            raise Exception(f'Waveform is out of spec.  Check times: {times}')


import serial
import serial.tools.list_ports as ports
from anasymod.enums import CtrlOps
from anasymod.base_config import BaseConfig
from anasymod.sim_ctrl.ctrlapi import CtrlApi
from anasymod.enums import ConfigSections
from anasymod.config import EmuConfig


class UARTCtrlApi(CtrlApi):
    """
    Start an interactive control interface to HW target for running regression tests or design exploration/debug.
    For FPGA/Emulators, as a pre-requisit, bitstream must have been created and programmed. Additionally any eSW
    necessary in the targeted system must have already been programmed.
    """
    def __init__(self, prj_cfg: EmuConfig):
        super().__init__()

        # Initialize control config
        self.cfg = Config(cfg_file=prj_cfg.cfg_file)

        self.vid_list = [1027]
        self.pid_list = [24592]
        self.port_list = []

    ### User Functions

    def sendline(self, line, timeout=float('inf')):
        """
        Send a single line in target shell specific language e.g. in tcl for tcl shell.
        :param line: Line that shall be send to/processed by shell
        :param timeout: Maximum time granted for operation to finish
        :return:
        """
        raise NotImplementedError("Base class was called to execute function")

    def source(self, script, timeout=float('inf')):
        """
        Source a script written language for targeted shell.
        :param script: Name/Path to script that shall be sourced
        :param timeout: Maximum time granted for operation to finish
        :return:
        """
        raise NotImplementedError("Base class was called to execute function")

    def refresh_param(self, name, timeout=30):
        """
        Refresh selected control parameter.
        :param name: Name of control parameter
        :param timeout: Maximum time granted for operation to finish
        :return:
        """
        raise NotImplementedError("Base class was called to execute function")

    def get_param(self, name, timeout=30):
        """
        Read value of a control parameter in design.
        :param name: Name of control parameter to be read
        :param timeout: Maximum time granted for operation to finish
        :return:
        """
        self._write(operation=CtrlOps.READ_PARAMETER, name=name)
        return self._read()

    def set_param(self, name, value, timeout=30):
        """
        Set value of a control parameter in design.
        :param name: Name of control parameter to be set
        :param value: Value of control parameter sto be set
        :param timeout: Maximum time granted for operation to finish
        :return:
        """
        self._write(operation=CtrlOps.WRITE_PARAMETER, name=name, value=value)
        if self._read():
            raise Exception(f"ERROR: Couldn't properly write: {name}={value} command to FPGA.")

    def set_var(self, name, value):
        """
        Define a variable in target shell environment.
        :param name: Name of variable that shall be set
        :param value: Value of variable that shall be set
        :return:
        """
        raise NotImplementedError("Base class was called to execute function")

    ### Utility Functions

    def _initialize(self):
        """
        Initialize the control interface, this is usually done after the bitstream was programmed successfully on the FPGA.
        :return:
        """
        if self.cfg.comport is None:  # run auto search for finding the correct COM port
            # find all available COM ports
            comports = [port for port in ports.comports(include_links=True)]
            # check if any COM port is compliant to known vids and pids and if so store the device_id
            for port in comports:
                if ((port.vid in self.vid_list) and (port.pid in self.pid_list)):
                    self.port_list.append(port.device)

            for port in self.port_list:
                try:
                    self.ctrl_handler = serial.Serial(port, int(self.cfg.baud_rate), timeout=self.cfg.timeout,
                                                      parity=self.cfg.parity, stopbits=self.cfg.stopbits,
                                                      bytesize=self.cfg.bytesize)
                    self.cfg.comport = port
                except:
                    pass
            if self.cfg.comport is None:
                raise Exception(
                    f"ERROR: No COM port could be opened to enable connection to FPGA, found ports were: {self.port_list}.")

        else:
            try:
                self.ctrl_handler = serial.Serial(self.cfg.comport, int(self.cfg.baud_rate), timeout=self.cfg.timeout,
                                                  parity=self.cfg.parity, stopbits=self.cfg.stopbits,
                                                  bytesize=self.cfg.bytesize)
            except:
                raise Exception(f"ERROR: Provided COM port: {self.cfg.comport} could not ne opened for communication.")

    def _setup_ctrl(self, server_addr):
        """
        Prepare instrumentation on the FPGA to allow interactive control.
        :param server_addr: Address of remote hardware server
        :return:
        """
        raise NotImplementedError("Base class was called to execute function")

    def _expect_prompt(self, timeout=float('inf')):
        """
        Wait for the shell used to transmit commands to provide a response.
        :param timeout: Maximum time granted for shell to open
        :return:
        """
        raise NotImplementedError("Base class was called to execute function")

    def _write(self, operation, name, value=None):
        # check is space is in any of the give input strings
        if ' ' in [operation, name, value]:
            raise Exception(f"Blanks in any of the provided argument strings;{operation}, {name}, {value}; sent via control interface are not allowed!")

        self.ctrl_handler.write((' '.join([str(operation), str(name), str(value) + '\r']).encode('utf-8')))
        self.ctrl_handler.flush()

    def _read(self, count=1):
        for idx in range(count):
            result = self.ctrl_handler.readline()

            if result not in ['', None]:
                return int(result.decode("utf-8").rstrip())
        raise Exception(f"ERROR: Couldn't read from FPGA after:{count} attempts.")


    def __del__(self):
        """
        Close connection to shell.
        :return:
        """
        pass


class Config(BaseConfig):
    """
    Container to store all config attributes.
    """
    def __init__(self, cfg_file):
        super().__init__(cfg_file=cfg_file, section=ConfigSections.FPGASIM)

        self.comport = None
        """ type(str) : Name of the COM port, that is attached to FPGA """

        self.baud_rate = 115200
        """ type(int): Baudrate used for communication with UART on FPGA """

        self.timeout = None
        """ type(int): Timeout in seconds used when waiting for a response during data transmission """

        self.parity = serial.PARITY_NONE
        """ type(str): Parity mode used for UART communication """

        self.stopbits=1
        """ type(int): Number of stop bits used when transmitting a package """

        self.bytesize=serial.EIGHTBITS
        """ type(int): Number of bits per transmitted package """

def main():
   ctrl = UARTCtrlApi(prj_cfg=EmuConfig(root='',cfg_file=''))
   ctrl.set_param(name=0, value=3)
   ctrl.set_param(name=1, value=4)
   print(ctrl.get_param(name=0))
   print(ctrl.get_param(name=1))

if __name__ == "__main__":
   main()


import os
import pytest
from pathlib import Path
from math import exp

root = Path(__file__).resolve().parent

from ..common import run_simulation, run_emulation, CommonArgParser, DEFAULT_SIMULATOR

def test_rc_sim(simulator_name=DEFAULT_SIMULATOR):
    run_simulation(root=root, simulator_name=simulator_name)

@pytest.mark.skipif(
    'FPGA_SERVER' not in os.environ,
    reason='The FPGA_SERVER environment variable must be set to run this test.'
)
def test_rc_emu(gen_bitstream=True, emulate=True):
    run_emulation(root=root, gen_bitstream=gen_bitstream,
                  emu_ctrl_fun=emu_ctrl_fun, emulate=emulate)

def emu_ctrl_fun(ctrl, v_in=1.0, n_steps=25, dt=0.1e-6, tau=1.0e-6, abs_tol=1e-3):
    # initialize values
    ctrl.stall_emu()
    ctrl.set_param(name='v_in', value=v_in)

    # reset emulator
    ctrl.set_reset(1)
    ctrl.set_reset(0)

    # walk through simulation values
    for _ in range(n_steps):
        # get readings
        ctrl.refresh_param('vio_0_i')
        v_out = ctrl.get_param('v_out')
        t_sim = ctrl.get_emu_time()

        # print readings
        print(f't_sim: {t_sim}, v_out: {v_out}')

        # check results
        meas_val = v_out
        expt_val = v_in*(1 - exp(-t_sim/tau))
        assert (expt_val - abs_tol) <= meas_val <= (expt_val + abs_tol), \
            'Measured value is out of range.'

        # wait a certain amount of time
        ctrl.sleep_emu(dt)

if __name__ == '__main__':
    CommonArgParser(sim_fun=test_rc_sim, emu_fun=test_rc_emu)



import os
import pytest
import serial
from pathlib import Path
from anasymod.analysis import Analysis

root = Path(__file__).resolve().parent

from ..common import DEFAULT_SIMULATOR

def test_1(simulator_name=DEFAULT_SIMULATOR):
    # run simulation
    ana = Analysis(input=root, simulator_name=simulator_name)
    ana.gen_sources()
    ana.simulate()

@pytest.mark.skipif(
    'FPGA_SERVER' not in os.environ,
    reason='The FPGA_SERVER environment variable must be set to run this test.'
)
def test_2():
    # build bitstream
    ana = Analysis(input=root)
    ana.set_target(target_name='fpga')  # set the active target to 'fpga'
    ana.build()

@pytest.mark.skipif(
    'FPGA_SERVER' not in os.environ,
    reason='The FPGA_SERVER environment variable must be set to run this test.'
)
def test_3():
    # build ELF
    ana = Analysis(input=root)
    ana.set_target(target_name='fpga')  # set the active target to 'fpga'
    ana.build_firmware()

@pytest.mark.skipif(
    'FPGA_SERVER' not in os.environ,
    reason='The FPGA_SERVER environment variable must be set to run this test.'
)
def test_4():
    # download program
    ana = Analysis(input=root)
    ana.set_target(target_name='fpga')  # set the active target to 'fpga'
    ana.program_firmware()

@pytest.mark.skipif(
    'FPGA_SERVER' not in os.environ,
    reason='The FPGA_SERVER environment variable must be set to run this test.'
)
def test_5():
    # run UART test
    ser = serial.Serial(
        port='/dev/ttyUSB2',
        baudrate=115200
    )

    # simple test
    ser.write(f'HELLO\n'.encode('utf-8'))
    out = ser.readline().decode('utf-8').strip()
    print(f'Got output: "{out}"')
    if out != 'Hello World!':
        raise Exception('Output mismatch.')

    # detailed test
    def run_test(a, b, mode, expct):
        # write inputs
        ser.write(f'SET_A {a}\n'.encode('utf-8'))
        ser.write(f'SET_B {b}\n'.encode('utf-8'))
        ser.write(f'SET_MODE {mode}\n'.encode('utf-8'))

        # get output
        ser.write('GET_C\n'.encode('utf-8'))
        out = ser.readline().decode('utf-8').strip()
        c = int(out)

        print(f'a={a}, b={b}, mode={mode} -> c={c} (expct={expct})')

        if c != expct:
            raise Exception('Output mismatch.')

    # try out different operating modes
    run_test(12, 34, 0, 46)
    run_test(45, 10, 1, 35)
    run_test(10, 44, 2, 34)
    run_test(3, 7, 3, 21)
    run_test(9, 1, 4, 4)
    run_test(9, 1, 5, 18)
    run_test(2, 32, 6, 8)
    run_test(3, 3, 7, 24)
    run_test(56, 78, 8, 42)

    # quit the program
    print('Quitting the program...')
    ser.write('EXIT\n'.encode('utf-8'))



from anasymod.sim_ctrl.datatypes import DigitalSignal

class TemplZynqGPIO:
    def __init__(self, ps_vlnv='xilinx.com:ip:processing_system7:5.5',
                 gpio_vlnv='xilinx.com:ip:axi_gpio:2.0', design_name='zynq_gpio'):
        # save settings
        self.design_name=design_name
        self.ps_vlnv = ps_vlnv
        self.gpio_vlnv = gpio_vlnv

        # save IO that are used by the emulator
        self.o_ctrl = DigitalSignal(name='o_ctrl', width=32, abspath=None)
        self.o_data = DigitalSignal(name='o_data', width=32, abspath=None)
        self.i_ctrl = DigitalSignal(name='i_ctrl', width=32, abspath=None)
        self.i_data = DigitalSignal(name='i_data', width=32, abspath=None)

        # generate text
        self.text = f'''
############################
# Create the block diagram #
############################

# Initialize

create_bd_design "{self.design_name}"

# Instantiate IPs

create_bd_cell -type ip -vlnv {self.ps_vlnv} processing_system7_0
create_bd_cell -type ip -vlnv {self.gpio_vlnv} axi_gpio_0
create_bd_cell -type ip -vlnv {self.gpio_vlnv} axi_gpio_1

# Configure IPs

set_property \\
    -dict [list \\
        CONFIG.C_IS_DUAL {{1}} \\
        CONFIG.C_ALL_OUTPUTS {{1}} \\
        CONFIG.C_ALL_INPUTS_2 {{1}} \\
    ] \\
    [get_bd_cells axi_gpio_0]

set_property \\
    -dict [list \\
        CONFIG.C_IS_DUAL {{1}} \\
        CONFIG.C_ALL_OUTPUTS {{1}} \\
        CONFIG.C_ALL_OUTPUTS_2 {{1}} \\
    ] \\
    [get_bd_cells axi_gpio_1]

# Create ports

create_bd_port -dir O -from 31 -to 0 o_ctrl 
create_bd_port -dir I -from 31 -to 0 o_data
create_bd_port -dir O -from 31 -to 0 i_ctrl 
create_bd_port -dir O -from 31 -to 0 i_data

# Wire up IPs

connect_bd_net \\
    [get_bd_pins processing_system7_0/FCLK_CLK0] \\
    [get_bd_pins processing_system7_0/M_AXI_GP0_ACLK]

connect_bd_net \\
    [get_bd_ports o_ctrl] \\
    [get_bd_pins axi_gpio_0/gpio_io_o]

connect_bd_net \\
    [get_bd_ports o_data] \\
    [get_bd_pins axi_gpio_0/gpio2_io_i]

connect_bd_net \\
    [get_bd_ports i_ctrl] \\
    [get_bd_pins axi_gpio_1/gpio_io_o]

connect_bd_net \\
    [get_bd_ports i_data] \\
    [get_bd_pins axi_gpio_1/gpio2_io_o]

# Apply automation    

apply_bd_automation \\
    -rule xilinx.com:bd_rule:processing_system7 \\
    -config {{ \\
        make_external "FIXED_IO, DDR" \\
        apply_board_preset "1" \\
        Master "Disable" \\
        Slave "Disable" \\
    }} \\
    [get_bd_cells processing_system7_0]

apply_bd_automation \\
    -rule xilinx.com:bd_rule:axi4 \\
    -config {{ \\
        Clk_master {{/processing_system7_0/FCLK_CLK0 (100 MHz)}} \\
        Clk_slave {{Auto}} \\
        Clk_xbar {{Auto}} \\
        Master {{/processing_system7_0/M_AXI_GP0}} \\
        Slave {{/axi_gpio_0/S_AXI}} \\
        intc_ip {{New AXI Interconnect}} \\
        master_apm {{0}}\\
    }} \\
    [get_bd_intf_pins axi_gpio_0/S_AXI]

apply_bd_automation \\
    -rule xilinx.com:bd_rule:axi4 \\
    -config {{ \\
        Clk_master {{/processing_system7_0/FCLK_CLK0 (100 MHz)}} \\
        Clk_slave {{Auto}} \\
        Clk_xbar {{Auto}} \\
        Master {{/processing_system7_0/M_AXI_GP0}} \\
        Slave {{/axi_gpio_1/S_AXI}} \\
        intc_ip {{New AXI Interconnect}} \\
        master_apm {{0}}\\
    }} \\
    [get_bd_intf_pins axi_gpio_1/S_AXI]

validate_bd_design
'''

    # save external IOs that need to be wired to the top
    EXT_IOS = [
        DigitalSignal(name='DDR_addr', width=15, abspath=None),
        DigitalSignal(name='DDR_ba', width=3, abspath=None),
        DigitalSignal(name='DDR_cas_n', width=1, abspath=None),
        DigitalSignal(name='DDR_ck_n', width=1, abspath=None),
        DigitalSignal(name='DDR_ck_p', width=1, abspath=None),
        DigitalSignal(name='DDR_cke', width=1, abspath=None),
        DigitalSignal(name='DDR_cs_n', width=1, abspath=None),
        DigitalSignal(name='DDR_dm', width=4, abspath=None),
        DigitalSignal(name='DDR_dq', width=32, abspath=None),
        DigitalSignal(name='DDR_dqs_n', width=4, abspath=None),
        DigitalSignal(name='DDR_dqs_p', width=4, abspath=None),
        DigitalSignal(name='DDR_odt', width=1, abspath=None),
        DigitalSignal(name='DDR_ras_n', width=1, abspath=None),
        DigitalSignal(name='DDR_reset_n', width=1, abspath=None),
        DigitalSignal(name='DDR_we_n', width=1, abspath=None),
        DigitalSignal(name='FIXED_IO_ddr_vrn', width=1, abspath=None),
        DigitalSignal(name='FIXED_IO_ddr_vrp', width=1, abspath=None),
        DigitalSignal(name='FIXED_IO_mio', width=54, abspath=None),
        DigitalSignal(name='FIXED_IO_ps_clk', width=1, abspath=None),
        DigitalSignal(name='FIXED_IO_ps_porb', width=1, abspath=None),
        DigitalSignal(name='FIXED_IO_ps_srstb', width=1, abspath=None)
    ]


# References:
# 1. https://www.xilinx.com/html_docs/xilinx2018_1/SDK_Doc/xsct/use_cases/xsdb_standalone_app_debug.html
# 2. https://github.com/Digilent/Arty-Z7-20-linux_bd/blob/master/sdk/.sdk/launch_scripts/xilinx_c-c%2B%2B_application_(system_debugger)/system_debugger_using_debug_video.elf_on_local.tcl
# 3. https://xilinx-wiki.atlassian.net/wiki/spaces/A/pages/18842240/Programming+QSPI+from+U-boot+ZC702

class TemplXSCTProgram:
    def __init__(self, sdk_path, bit_path, hw_path, tcl_path, cpu_filter='"ARM*#0"',
                 sw_name='sw', program_fpga=True, reset_system=True):

        # initialize text
        self.text = ''

        # apply commands

        self.connect()

        self.select_cpu(cpu_filter)

        if reset_system:
            self.reset_system()

        if program_fpga:
            self.program_fpga(bit_path)

        self.loadhw(hw_path)

        self.init_cpu(tcl_path)

        self.download(str(sdk_path / sw_name / 'Debug' / sw_name) + '.elf')

        self.run()

        self.puts("Program started.")

    def line(self, s='', nl='\n'):
        self.text += f'{s}{nl}'

    def puts(self, s):
        self.line(f'puts "{s}"')

    def connect(self):
        self.puts('Connecting to the HW server...')
        self.line('connect')
        self.line()

    def select_cpu(self, cpu_filter):
        self.puts('Selecting the CPU...')
        self.line(f'targets -set -filter {{name =~ {cpu_filter}}}')
        self.line()

    def reset_system(self):
        self.puts('Resetting the system...')
        self.line('rst')
        self.line()

    def program_fpga(self, bit_path):
        self.puts('Programming the FPGA...')
        self.line(f'fpga "{bit_path}"')
        self.line()

    def loadhw(self, hw_path):
        self.puts('Setting up the debugger...')
        self.line(f'loadhw "{hw_path}"')
        self.line()

    def init_cpu(self, tcl_path):
        self.puts('Initializing the processor...')
        self.line(f'source "{tcl_path}"')
        self.line('ps7_init')
        self.line('ps7_post_config')
        self.line()

    def download(self, elf_path):
        self.puts('Downloading the program...')
        self.line(f'dow "{elf_path}"')
        self.line()

    def run(self):
        self.puts('Starting the program...')
        self.line('con')
        self.line()



import os
from anasymod.sim_ctrl.ctrlinfra import ControlInfrastructure
from anasymod.structures.module_uartsimctrl import ModuleUARTSimCtrl
from anasymod.structures.module_regmapsimctrl import ModuleRegMapSimCtrl
from anasymod.sources import VerilogSource, BDFile
from anasymod.files import get_from_anasymod
from anasymod.structures.structure_config import StructureConfig

class UARTControlInfrastructure(ControlInfrastructure):
    def __init__(self, prj_cfg):
        super().__init__(prj_cfg=prj_cfg)

        # Initialize internal variables
        self._simctrlregmap_path = os.path.join(prj_cfg.build_root, 'gen_ctrlregmap.sv')

        # Program Zynq PS for UART access
        self._program_zynq_ps()

        #TODO: add path to elf file and add structure for storing ctrl ifc dependent files,
        # also including the BD; later test if .tcl script for creating BD would be
        # beneficial/at least there should be a script for creating the .bd file for a
        # new Vivado version, also add binary path to xsct interface

    def gen_ctrlwrapper(self, str_cfg: StructureConfig, content):
        """
        Generate RTL design for control infrastructure. This will generate the register map, add the
        block diagram including the zynq PS and add the firmware running on the zynq PS.
        """

        # Generate simulation control wrapper and add to target sources
        with (open(self._simctrlwrap_path, 'w')) as ctrl_file:
           ctrl_file.write(ModuleUARTSimCtrl(scfg=str_cfg).render())

        content.verilog_sources += [VerilogSource(files=self._simctrlwrap_path, name='simctrlwrap')]

    def gen_ctrl_infrastructure(self, str_cfg: StructureConfig, content):
        """
        Generate RTL design for FPGA specific control infrastructure, depending on the interface
        selected for communication. For UART_ZYNQ control a register map, ZYNQ CPU SS block
        diagram and the firmware running on the zynq PS need to be handled.
        """

        # Generate register map according to IO settings stored in structure config and add to target sources
        with (open(self._simctrlregmap_path, 'w')) as ctrl_file:
           ctrl_file.write(ModuleRegMapSimCtrl(scfg=str_cfg).render())

        content.verilog_sources += [VerilogSource(files=self._simctrlregmap_path, name='simctrlregmap')]

        #TODO: Add firmware part here -> generate FW if needed and add it to target sources

    def _program_zynq_ps(self):
        """
        Program UART control application to Zynq PS to enable UART control interface.
        """

        pass

    def add_ip_cores(self, scfg, ip_dir):
        """
        Configures and adds IP cores that are necessary for selected IP cores. No IP core is
        configured and added, so this just returns an empty list.
        :return rendered template for configuring a vio IP core
        """

        return []